from wiliot_deployment_tools.gw_certificate.interface.pkt_generator import PktGenerator
import random
import numpy as np

class BrgArray:
    def __init__(self, num_brgs=3):
        assert num_brgs > 1, 'BrgArray cannot be smaller than 1!'
        self.brg_list = [PktGenerator() for brg in range(num_brgs)]
        self.primary_brg = self.brg_list[0]
        self.secondary_brgs = self.brg_list[1:]
    
    def get_new_pkt_pairs(self) -> list:
        pkts = []
        new_pkt_primary = self.primary_brg.get_new_packet_pair()
        pkts.append(new_pkt_primary)
        for brg in self.secondary_brgs:
            brg.generate_si_from_pkt_id(self.primary_brg.pkt_id_int)
            brg.randomize_rssi_nfpkt()
            brg.data_packet = new_pkt_primary['data_packet']
            pkts.append(brg.get_existing_packet_pair())
        for idx, pkt in enumerate(pkts):
            pkt.update({'bridge_id': self.brg_list[idx].bridge_id})
        return pkts
    
    def get_scattered_pkt_pairs(self, delay):
        assert delay / len(self.brg_list) >= 10, 'Cannos get scattered PKT pairs for parameters! decrease num of brgs or increase delay'
        start_times = sorted(random.sample(list(np.arange(0, delay-10+1, 10)), k=len(self.brg_list)))
        start_times.append((start_times[0] + delay))
        pkts = self.get_new_pkt_pairs()
        
        for idx, pkt in enumerate(pkts):
            # minimum time between UART packet commands = 10ms
            pkt['time_delay'] = max(start_times[idx+1] - start_times[idx], 10)
        return pkts


import time
import os
from wiliot_deployment_tools.api.extended_api import ExtendedEdgeClient
from wiliot_deployment_tools.common.analysis_data_bricks import initialize_logger
from wiliot_deployment_tools.common.debug import debug_print
from wiliot_core.utils.utils import WiliotDir
from wiliot_deployment_tools.gw_certificate.interface.ble_simulator import BLESimulator
from wiliot_deployment_tools.gw_certificate.interface.mqtt import MqttClient
from wiliot_deployment_tools.gw_certificate.tests.coupling import CouplingTest
from wiliot_deployment_tools.interface.uart_ports import get_uart_ports

class GWCertificateError(Exception):
    pass

class GWCertificate:
    def __init__(self, gw_id, api_key, owner_id, env='prod', random=False, legacy=False):
        self.env_dirs = WiliotDir()
        self.certificate_dir = os.path.join(self.env_dirs.get_wiliot_root_app_dir(), 'gw-certificate')
        self.env_dirs.create_dir(self.certificate_dir)
        self.logger_filename = initialize_logger(self.certificate_dir)
        self.gw_id = gw_id
        self.owner_id = owner_id
        self.edge = ExtendedEdgeClient(api_key, owner_id, env=env)
        self.mqttc = MqttClient(gw_id, owner_id, f'{self.certificate_dir}/{self.logger_filename}_mqtt.log')
        self.uart_comports = get_uart_ports()
        if random:
            debug_print('Randomizing Test!')
        self.randomize = random
        if legacy:
            debug_print('Working in LEGACY DevMode!')
        self.legacy = legacy
        debug_print(f'UART Ports:{self.uart_comports}')
        if len(self.uart_comports) < 1:
            raise GWCertificateError('A Wiliot GW needs to be connected to USB!')
        self.ble_sim = BLESimulator(self.uart_comports[0])
        if not self.ble_sim.check_version_for_sim():
            raise GWCertificateError('USB GW Version not compatible with BLE Simulator!')
        # self.sniffer = SnifferClass(self.uart_comports[1])
        self.tests = [CouplingTest(**self.__dict__)]
    
    def run_tests(self):
        for test in self.tests:
            test.run()

# TODO - send HB and CFG 
# TODO - UnCoupled Mode



import pandas as pd
import tabulate
from wiliot_deployment_tools.api.extended_api import ExtendedEdgeClient
from wiliot_deployment_tools.gw_certificate.interface.ble_simulator import BLESimulator
from wiliot_deployment_tools.gw_certificate.interface.mqtt import MqttClient

PASS_STATUS = {True: 'PASS', False: 'FAIL'}

class GenericTest:
    def __init__(self, mqttc: MqttClient, ble_sim: BLESimulator, edge: ExtendedEdgeClient, gw_id, owner_id, **kwargs):
        self.mqttc = mqttc
        self.ble_sim = ble_sim
        self.edge = edge
        self.report = ''
        self.test_pass = False
    
    def add_to_test_report(self, report):
        self.report += '\n' + report


import binascii
import datetime
import pandas as pd
import os

import pkg_resources
from wiliot_deployment_tools.ag.ut_defines import BRIDGE_ID, NFPKT, PAYLOAD, RSSI, TIMESTAMP

from wiliot_deployment_tools.ag.wlt_types import GenericV7, WltPkt
from wiliot_deployment_tools.ag.wlt_types_ag import GROUP_ID_SIDE_INFO, Hdr, SideInfo
from wiliot_deployment_tools.ag.wlt_types_data import DataPacket
from wiliot_deployment_tools.common.debug import debug_print

EMPTY_PKT = '000000000000000000000000000000000000000000000000'
class PktGenerator:
    def __init__(self):
        self.bridge_id = 'FFFFFFFFFFFF'
        self.adva = 'FFFFFFFFFFFF'
        self.generate_random_bridge_id()
        
        # Data packet init
        self.data_packet = DataPacket()
        self.pkt_id_int = self.data_packet.generic.pkt_id
        self.pkt_id_bytes = self.pkt_id_int.to_bytes(4, 'big').hex()
            # (int.from_bytes(binascii.unhexlify(self.pkt_id), 'big') +1)        
        # SI packet init
        self.si_packet = self.generate_si_from_pkt_id(self.pkt_id_int)
        self.rssi = self.si_packet.generic.rssi
        self.nfpkt = self.si_packet.generic.nfpkt
                
    def generate_si_from_pkt_id(self, pkt_id_int):
        self.si_packet = WltPkt(hdr=Hdr(group_id=GROUP_ID_SIDE_INFO), generic=SideInfo(brg_mac=int.from_bytes(binascii.unhexlify(self.bridge_id), 'big'), pkt_id=pkt_id_int))
        return self.si_packet
    
    def generate_random_bridge_id(self):
        self.bridge_id = os.urandom(6).hex().upper()
        
    def increment_pkt_id(self):
        self.pkt_id_int = self.pkt_id_int + 1
        self.pkt_id_bytes = self.pkt_id_int.to_bytes(4, 'big').hex()
        self.data_packet.generic.pkt_id = self.pkt_id_int
        self.si_packet.generic.pkt_id = self.pkt_id_int
    
    def randomize_data_packet(self):
        # randomize packet ID
        self.pkt_id_int = int.from_bytes(os.urandom(4), 'big')
        self.pkt_id_bytes = self.pkt_id_int.to_bytes(4, 'big').hex()
        self.data_packet.generic.pkt_id = self.pkt_id_int
        self.si_packet.generic.pkt_id = self.pkt_id_int
        # randomize packet payload
        self.data_packet.generic.payload = int.from_bytes(os.urandom(20), 'big')
        
    def randomize_rssi_nfpkt(self):
        self.rssi = int.from_bytes(os.urandom(1), 'big')
        self.nfpkt = int.from_bytes(os.urandom(2), 'big')
        self.si_packet.generic.rssi = self.rssi
        self.si_packet.generic.nfpkt = self.nfpkt
        return self.rssi, self.nfpkt
    
    def set_rssi_nfpkt(self, rssi, nfpkt):
        self.rssi = rssi
        self.nfpkt = nfpkt
        self.si_packet.generic.rssi = self.rssi
        self.si_packet.generic.nfpkt = self.nfpkt
    
    def get_existing_packet_pair(self) -> dict:
        return {'data_packet': self.data_packet, 'si_packet': self.si_packet}
    
    def get_new_packet_pair(self) -> dict: 
        #self.randomize_pkt_id()
        self.randomize_data_packet()
        self.randomize_rssi_nfpkt()
        return self.get_existing_packet_pair()
        
    def get_expected_mqtt(self):
        """generates expected MQTT packet"""
        timestamp = int(datetime.datetime.now().timestamp()*1000)
        expected = {TIMESTAMP: timestamp, BRIDGE_ID: self.bridge_id, NFPKT: self.nfpkt, RSSI: self.rssi, PAYLOAD: self.data_packet.dump()[4:]}
        return expected
